/*========================================================================================================
 * ArcadeDrive.cpp
 *========================================================================================================
 *
 * File Description:
 *
 * This implements the IDrive interface to drive the robot using joysticks in a arcade
 * drive mode.
 *
 * Thus, the left axis will control speed (throttle) and the right axis will control direction
 * (steering). The speed control (throttle) will get a cubic profile while the steer control
 * will be direct (linear).
 *
 * Then, the left and right speeds are set by using the arcade drive equations where
 * the left side adds the steer value to the throttle value and the right side
 * subtracts the steer value from the trottle.
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/

// Team 302 includes
#include <ArcadeDrive.h>            // This class
#include <IChassis.h>               // Class the control the drive motors
#include <IChassisFactory.h>        // Class constructs the correct chassis
#include <DragonStick.h>            // Class for the driver gamepad
#include <OperatorInterface.h>      // Class that owns the driver gamepad

//--------------------------------------------------------------------
// Method:      ArcadeDrive <<constructor>>
// Description: This method creates and initializes the objects
//--------------------------------------------------------------------
ArcadeDrive::ArcadeDrive() : IDrive()
{
    Init();
}

//--------------------------------------------------------------------
// Method:      Init
// Description: This method will initialize the values.
// Returns:     void
//--------------------------------------------------------------------
void ArcadeDrive::Init()
{
    CommonInit( ARCADE_DRIVE_THROTTLE, ARCADE_DRIVE_STEER );
}

//--------------------------------------------------------------------
// Method:      CommonInit
// Description: This method to initialize the axis being used.
// Returns:     void
//--------------------------------------------------------------------
void ArcadeDrive::CommonInit
(
    FUNCTION_IDENTIFIER     throttle,       // <I> - axis being used for throtte
    FUNCTION_IDENTIFIER     steer           // <I> - axis being used for steer
)
{
    m_throttle = throttle;
    m_steer    = steer;

    m_oi    = OperatorInterface::GetInstance();
    m_oi->SetAxisProfile( throttle, CUBIC_SCALING );
    m_oi->SetAxisProfile( steer, LINEAR_SCALING );

    // Set to Coast Mode
    m_chassis = IChassisFactory::GetInstance()->GetIChassis();
    m_chassis->SetCoastMode();
}

//--------------------------------------------------------------------
// Method:      HasTarget
// Description: This indicates if this drive mode doesn't have a target
// Returns:     false = doesn't have a target
//--------------------------------------------------------------------
bool ArcadeDrive::HasTarget() const
{
    return false;
}


//--------------------------------------------------------------------
// Method:      SetTarget
// Description: This sets the target for the drive class.
// Returns:     void
//--------------------------------------------------------------------
void ArcadeDrive::SetTarget
(
    float   target
)
{
}

//--------------------------------------------------------------------
// Method:      DrivePeriodic
// Description: This method will be called during the periodic loop
//              to drive the robot.  It will interact with the joysticks
//              to drive in arcade drive mode.
// Returns:     false = this mode is never done
//--------------------------------------------------------------------
bool ArcadeDrive::DrivePeriodic() const
{
    // TODO:    Read the joystick values and calculate (using the formula above)
    //          the values to send to the motors.
    //          Send these values to the chassis.
    //
    //          Try reversing the throttle and the steer sticks.  Does the
    //          formula change?
    //
    // Read the desired motor speeds
    float steer      = m_oi->GetRawAxis( m_steer );
    float throttle   = m_oi->GetRawAxis( m_throttle );

    // Convert to using arcade equations to give motor speeds on
    // both sides of the robot.
    float leftSpeed  = 0.0;
    float rightSpeed = 0.0;

    // Set the motors
    m_chassis->SetMotorSpeeds( leftSpeed, rightSpeed );

    return false;
}


//--------------------------------------------------------------------
// Method:      GetIdentifier
// Description: This gets the name that identifies the drive mode..
// Returns:     const char*         identifier for the drive
//--------------------------------------------------------------------
const char* ArcadeDrive::GetIdentifier() const
{
    return "Arcade Drive";
}