/*========================================================================================================
 * ArcadeDrive.h
 *========================================================================================================
 *
 * File Description:
 *
 * This implements the IDrive interface to drive the robot using joysticks in a arcade
 * drive mode.
 *
 * Thus, the left axis will control speed (throttle) and the right axis will control direction
 * (steering). The speed control (throttle) will get a cubic profile while the steer control
 * will be direct (linear).
 *
 * Then, the left and right speeds are set by using the arcade drive equations where
 * the left side adds the steer value to the throttle value and the right side
 * subtracts the steer value from the trottle.
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/

#ifndef SRC_ARCADEDRIVE_H_
#define SRC_ARCADEDRIVE_H_


//  Team 302 includes
#include <IDrive.h>             // Interface being implemented
#include <IChassis.h>           // Class that controls the motors on the drive chassis
#include <DragonStick.h>        // Driver Game pad
#include <OperatorInterface.h>  // DragonStick Mapping to the functions

class ArcadeDrive: public IDrive
{
    public:
        //--------------------------------------------------------------------
        // Method:      ArcadeDrive <<constructor>>
        // Description: This method creates and initializes the objects
        //--------------------------------------------------------------------
        ArcadeDrive();      // constructor

        //--------------------------------------------------------------------
        // Method:      Init
        // Description: This method will initialize the values.
        // Returns:     void
        //--------------------------------------------------------------------
        void Init() override;

        //--------------------------------------------------------------------
        // Method:      HasTarget
        // Description: This indicates if this drive mode has a target
        //              such as time, distance, etc.
        // Returns:     true  = has a target
        //              false = doesn't have a target
        //--------------------------------------------------------------------
        bool HasTarget() const override;


        //--------------------------------------------------------------------
        // Method:      SetTarget
        // Description: This sets the target for the drive class.
        // Returns:     void
        //--------------------------------------------------------------------
        void SetTarget
        (
            float   target
        ) override;

        //--------------------------------------------------------------------
        // Method:      DrivePeriodic
        // Description: This method will be called during the periodic loop
        //              to drive the robot.  It will interact with the joysticks
        //              to drive in arcade drive mode.
        // Returns:     false = this mode is never done
        //--------------------------------------------------------------------
        bool DrivePeriodic() const override;

        //--------------------------------------------------------------------
        // Method:      GetIdentifier
        // Description: This gets the name that identifies the drive mode..
        // Returns:     const char*         identifier for the drive
        //--------------------------------------------------------------------
        const char* GetIdentifier() const override;

        //--------------------------------------------------------------------
        // Method:      ~ArcadeDrive <<destructor>>
        // Description: This method is called to delete these objects
        //--------------------------------------------------------------------
        virtual ~ArcadeDrive() = default; // destructor

    protected:
        //--------------------------------------------------------------------
        // Method:      CommonInit
        // Description: This method to initialize the axis being used.
        // Returns:     void
        //--------------------------------------------------------------------
        void CommonInit
        (
            FUNCTION_IDENTIFIER     throttle,       // <I> - axis being used for throtte
            FUNCTION_IDENTIFIER     steer           // <I> - axis being used for steer
        );

    private:
        // Attributes
        OperatorInterface*      m_oi;
        IChassis*               m_chassis;

        FUNCTION_IDENTIFIER     m_throttle;
        FUNCTION_IDENTIFIER     m_steer;

        // Default methods we don't want the compiler to implement for us
        ArcadeDrive( const ArcadeDrive& ) = delete;
        ArcadeDrive& operator= ( const ArcadeDrive& ) = delete;
        explicit ArcadeDrive( ArcadeDrive* other ) = delete;


};

#endif /* SRC_ARCADEDRIVE_H_ */
