/*========================================================================================================
 * SoftwareTestChassis.cpp
 *========================================================================================================
 *
 * File Description:
 *
 * This file interacts with the hardware (motors and sensors) used to drive the robot.
 *========================================================================================================
 *  Modifications:
 *  Date			Name			Description of Change
 *  18-Oct-2016		Joe Witcpalek	Created File
 *========================================================================================================*/

// WPILib includes
#include <CANTalon.h>						// Motor Controllers

// Team 302 includes
#include <SoftwareTestChassis.h>			// This class'
#include <IChassis.h>						// Interface this class implements'
#include <LimitValue.h>						// Contains utility to force values within the valid range
#include <RobotMap.h>						// Contains the CAN IDs


//----------------------------------------------------------------------------------
// Method:		SoftwareTestChassis <<Constructor>>
// Description:	Create the motor controllers and set whether they are inverted or not.
//----------------------------------------------------------------------------------
SoftwareTestChassis::SoftwareTestChassis() : IChassis(),
					 	 	 	 	 	 	 m_frontLeftMotor( nullptr ) ,
											 m_frontRightMotor( nullptr ),
											 m_backLeftMotor( nullptr ),
											 m_backRightMotor( nullptr ),
											 m_frontLeftEncResetValue( 0 ),
											 m_frontRightEncResetValue( 0 ),
											 m_backLeftEncResetValue( 0 ),
											 m_backRightEncResetValue( 0 )
{
	// Create each drive motor and set whether it is inverted or not
	m_frontLeftMotor  = new CANTalon( FRONT_LEFT_DRIVE_MOTOR );
	m_frontLeftMotor->SetInverted( IS_FRONT_LEFT_DRIVE_MOTOR_INVERTED );

	m_frontRightMotor = new CANTalon( FRONT_RIGHT_DRIVE_MOTOR );
	m_frontRightMotor->SetInverted( IS_FRONT_RIGHT_DRIVE_MOTOR_INVERTED );

	m_backLeftMotor   = new CANTalon( BACK_LEFT_DRIVE_MOTOR );
	m_backLeftMotor->SetInverted( IS_BACK_LEFT_DRIVE_MOTOR_INVERTED );

	m_backRightMotor  = new CANTalon( BACK_RIGHT_DRIVE_MOTOR );
	m_backRightMotor->SetInverted( IS_BACK_RIGHT_DRIVE_MOTOR_INVERTED );

	ResetDistance();
}


//----------------------------------------------------------------------------------
// Method:		SetMotorSpeeds
// Description:	This will set the motor speeds for the left and right side of the
//				robot.  If a value is specified that is outside the range, the value
//				used will be the closest bounding value (e.g. if 2.0 is passed in and
//				the range is -1.0 to 1.0, 1.0 will be used.
// Returns:		void
//----------------------------------------------------------------------------------
void SoftwareTestChassis::SetMotorSpeeds
(
	const float leftSideSpeeds,	// <I> - Speed for the left side of the robot to (range -1.0 to 1.0)
	const float rightSideSpeeds	// <I> - Speed for the right side of the robot (range -1.0 to 1.0)
) const
{
	// Make sure the left side speed is within range and then set both left motors to this speed
	float leftSpeed = LimitValue::ForceInRange( leftSideSpeeds, -1.0, 1.0 );
	m_frontLeftMotor->Set( leftSpeed );
	m_backLeftMotor->Set( leftSpeed );

	// Make sure the right side speed is within range and then set both right motors to this speed
	float rightSpeed = LimitValue::ForceInRange( rightSideSpeeds, -1.0, 1.0 );
	m_frontRightMotor->Set( rightSpeed );
	m_backRightMotor->Set( rightSpeed );
}

//----------------------------------------------------------------------------------
// Method:		ResetDistance
// Description:	This will reset the distance, so calls to get the distance will
//				relative to this distance.
// Returns:		void
//----------------------------------------------------------------------------------
void SoftwareTestChassis::ResetDistance()
{
	m_frontLeftEncResetValue = m_frontLeftMotor->GetEncPosition();
	m_frontRightEncResetValue = m_frontRightMotor->GetEncPosition();
	m_backLeftEncResetValue = m_backLeftMotor->GetEncPosition();
	m_backRightEncResetValue = m_backRightMotor->GetEncPosition();
}

//----------------------------------------------------------------------------------
// Method:		GetLeftSideDistance
// Description:	This will return the distance the left side has traveled since the
//				last time ResetDistance() was called.
// Returns:		float	distance in feet the left side of the robot has travelled
//						(positive values are forward and negative values are backward)
//----------------------------------------------------------------------------------
float SoftwareTestChassis::GetLeftSideDistance() const
{
	int frontEncoderCounts = m_frontLeftMotor->GetEncPosition() - m_frontLeftEncResetValue;
	int rearEncoderCounts  = m_backLeftMotor->GetEncPosition()  - m_backLeftEncResetValue;
	return ConvertEncoderCountsToDistance( frontEncoderCounts, rearEncoderCounts );
}

//----------------------------------------------------------------------------------
// Method:		GetRightSideDistance
// Description:	This will return the distance the right side has traveled since the
//				last time ResetDistance() was called.
// Returns:		float	distance in feet the right side of the robot has travelled
//						(positive values are forward and negative values are backward)
//----------------------------------------------------------------------------------
float SoftwareTestChassis::GetRightSideDistance() const
{
	int frontEncoderCounts = m_frontRightMotor->GetEncPosition() - m_frontRightEncResetValue;
	int rearEncoderCounts  = m_backRightMotor->GetEncPosition()  - m_backRightEncResetValue;
	return ConvertEncoderCountsToDistance( frontEncoderCounts, rearEncoderCounts );
}

//----------------------------------------------------------------------------------
// Method:		ConvertEncoderCountsToDistance
// Description:	This will average the counts and then convert that value to feet
// Returns:		float	distance in feet (positive values are forward and negative
//						values are backward)
//----------------------------------------------------------------------------------
float SoftwareTestChassis::ConvertEncoderCountsToDistance
(
	int 	frontCounts,		// <I> - encoder counts for the front wheel
	int		rearCounts			// <I> - encoder counts for the rear wheel
) const
{
	float avgDist = static_cast<float>( frontCounts + rearCounts ) / 2.0;
	return ( avgDist * ENCODER_DISTANCE_CONVERSION);
}


//----------------------------------------------------------------------------------
// Method:		GetLeftSideVelocity
// Description:	This will return the velocity the left side of the robot
// Returns:		float	velocity in feet per second the left side of the robot is moving
//						(positive values are forward and negative values are backward)
//----------------------------------------------------------------------------------
float SoftwareTestChassis::GetLeftSideVelocity() const
{
	int frontEncoderCounts = m_frontLeftMotor->GetEncVel();
	int rearEncoderCounts  = m_backLeftMotor->GetEncVel();

	return ConvertEncoderCountsToVelocity( frontEncoderCounts, rearEncoderCounts );
}

//----------------------------------------------------------------------------------
// Method:		GetRightSideVelocity
// Description:	This will return the velocity the right side of the robot
// Returns:		float	velocity in feet per second the right side of the robot is moving
//						(positive values are forward and negative values are backward)
//----------------------------------------------------------------------------------
float SoftwareTestChassis::GetRightSideVelocity() const
{
	int frontEncoderCounts = m_frontRightMotor->GetEncVel();
	int rearEncoderCounts  = m_backRightMotor->GetEncVel();

	return ConvertEncoderCountsToVelocity( frontEncoderCounts, rearEncoderCounts );
}

//----------------------------------------------------------------------------------
// Method:		ConvertEncoderCountsToVelocity
// Description:	This will average the counts and then convert that value to feet per second
// Returns:		float	velocity in feet per second the robot is moving
//						(positive values are forward and negative values are backward)
//----------------------------------------------------------------------------------
float SoftwareTestChassis::ConvertEncoderCountsToVelocity
(
	int 	frontCounts,		// <I> - encoder counts for the front wheel
	int		rearCounts			// <I> - encoder counts for the rear wheel
) const
{
	float avgDist = static_cast<float>( frontCounts + rearCounts ) / 2.0;
	return ( avgDist * ENCODER_VELOCITY_CONVERSION);
}

//----------------------------------------------------------------------------------
// Method:		SetBrakeMode
// Description:	This will set the drive motors to brake mode
// Returns:		void
//----------------------------------------------------------------------------------
void SoftwareTestChassis::SetBrakeMode() const
{
	m_frontLeftMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Brake );
	m_frontRightMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Brake );
	m_backLeftMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Brake );
	m_backRightMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Brake );
}


//----------------------------------------------------------------------------------
// Method:		SetCoastMode
// Description:	This will set the drive motors to coast mode
// Returns:		void
//----------------------------------------------------------------------------------
void SoftwareTestChassis::SetCoastMode() const
{
	m_frontLeftMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Coast );
	m_frontRightMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Coast );
	m_backLeftMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Coast );
	m_backRightMotor->ConfigNeutralMode( CANSpeedController::kNeutralMode_Coast );
}

//----------------------------------------------------------------------------------
// Method:		~SoftwareTestChassis <<Destructor>>
// Description:	Delete the motor controllers
//----------------------------------------------------------------------------------
SoftwareTestChassis::~SoftwareTestChassis()
{
	delete m_frontLeftMotor;
	m_frontLeftMotor = nullptr;

	delete m_frontRightMotor;
	m_frontRightMotor = nullptr;

	delete m_backLeftMotor;
	m_backLeftMotor = nullptr;

	delete m_backRightMotor;
	m_backRightMotor = nullptr;
}

