/*========================================================================================================
 * TankDrive.h
 *========================================================================================================
 *
 * File Description:
 *
 * This implements the IDrive interface to drive the robot using joysticks in a tank
 * drive mode.
 *
 * Thus, the left axis will control the left side of the robot and the right axis will
 * control the right side of the robot.  Turning is achieve by pushing the axis in opposite
 * directions (e.g. right stick forward and left stick backward will turn left).
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/
#ifndef SRC_TANKDRIVE_H_
#define SRC_TANKDRIVE_H_


//  Team 302 includes
#include <IDrive.h>             // Interface being implemented
#include <IChassis.h>           // Class that controls the motors on the drive chassis
#include <DragonStick.h>        // Driver Game pad
#include <OperatorInterface.h>  // DragonStick Mapping to the functions


class TankDrive: public IDrive
{
    public:
        //--------------------------------------------------------------------
        // Method:      TankDrive <<Constructor>>
        // Description: This method is called to create/initialize this class
        //--------------------------------------------------------------------
        TankDrive();                    // constructor

        //--------------------------------------------------------------------
        // Method:      Init
        // Description: This method will initialize the values.
        // Returns:     void
        //--------------------------------------------------------------------
        void Init() override;

        //--------------------------------------------------------------------
        // Method:      HasTarget
        // Description: This indicates if this drive mode has a target
        //              such as time, distance, etc.
        // Returns:     true  = has a target
        //              false = doesn't have a target
        //--------------------------------------------------------------------
        bool HasTarget() const override;


        //--------------------------------------------------------------------
        // Method:      SetTarget
        // Description: This sets the target for the drive class.
        // Returns:     void
        //--------------------------------------------------------------------
        void SetTarget
        (
            float   target
        ) override;

        //--------------------------------------------------------------------
        // Method:      DrivePeriodic
        // Description: This method will be called during the periodic loop
        //              to drive the robot.  It will interact with the joysticks
        //              to drive in tank drive mode.
        // Returns:     false = this mode is never done
        //--------------------------------------------------------------------
        bool DrivePeriodic() const override;

        //--------------------------------------------------------------------
        // Method:      GetIdentifier
        // Description: This gets the name that identifies the drive mode..
        // Returns:     const char*         identifier for the drive
        //--------------------------------------------------------------------
        const char* GetIdentifier() const override;

        //--------------------------------------------------------------------
        // Method:      ~TankDrive <<destructor>>
        // Description: This method is called to delete these objects
        //--------------------------------------------------------------------
        virtual ~TankDrive() = default;

    private:
        // Attributes
        OperatorInterface*      m_oi;
        IChassis*               m_chassis;

        // Default methods we don't want the compiler to implement for us
        TankDrive( const TankDrive& ) = delete;
        TankDrive& operator= ( const TankDrive& ) = delete;
        explicit TankDrive( TankDrive* other ) = delete;

};

#endif /* SRC_TANKDRIVE_H_ */
