/*========================================================================================================
 * TriggerDrive.cpp
 *========================================================================================================
 *
 * File Description:
 *
 * This implements the IDrive interface to drive the robot using the triggers and bumpers to drive in a tank
 * mode.
 *
 * The left trigger will control the left side of the robot and the right trigger will
 * control the right side of the robot.  Pressing the bumper above the trigger will negate the input.
 * Turning is achieve by making the sides of the robot go in opposite directions (e.g. right side
 * goes forward and the left side in reverse).
 *========================================================================================================
 *  Modifications:
 *  Date            Name            Description of Change
 *  18-Oct-2016     Joe Witcpalek   Created File
 *========================================================================================================*/

// Team 302 includes
#include <TriggerDrive.h>           // This class
#include <IChassis.h>               // Class the control the drive motors
#include <IChassisFactory.h>        // Class constructs the correct chassis
#include <OperatorInterface.h>      // Class that owns the driver gamepad


//--------------------------------------------------------------------
// Method:      TriggerDrive <<Constructor>>
// Description: This method is called to create/initialize this class
//--------------------------------------------------------------------
TriggerDrive::TriggerDrive() : IDrive(),
                               m_oi( nullptr ),
                               m_chassis( nullptr )
{
    Init();
}

//--------------------------------------------------------------------
// Method:      Init
// Description: This method will initialize the values.
// Returns:     void
//--------------------------------------------------------------------
void TriggerDrive::Init()
{
    // Get the two axis that will be used for tank drive
    // Set their profile to be Cubed
    m_oi = OperatorInterface::GetInstance();
    m_oi->SetAxisProfile( LEFT_TRIGGER_DRIVE_CONTROL, CUBIC_SCALING );
    m_oi->SetAxisProfile( RIGHT_TRIGGER_DRIVE_CONTROL, CUBIC_SCALING );

    m_chassis = IChassisFactory::GetInstance()->GetIChassis();
    m_chassis->SetCoastMode();
}


//--------------------------------------------------------------------
// Method:      HasTarget
// Description: This indicates if this drive mode doesn't have a target
// Returns:     false = doesn't have a target
//--------------------------------------------------------------------
bool TriggerDrive::HasTarget() const
{
    return false;
}


//--------------------------------------------------------------------
// Method:      SetTarget
// Description: This sets the target for the drive class.
// Returns:     void
//--------------------------------------------------------------------
void TriggerDrive::SetTarget
(
    float   target
)
{
}

//--------------------------------------------------------------------
// Method:      DrivePeriodic
// Description: This method will be called during the periodic loop
//              to drive the robot.  It will interact with the joysticks
//              to drive in tank drive mode.
// Returns:     false = this mode is never done
//--------------------------------------------------------------------
bool TriggerDrive::DrivePeriodic() const
{
    // Determine the desired motor speeds
    float leftSpeed  = m_oi->GetRawAxis( LEFT_TRIGGER_DRIVE_CONTROL );
    leftSpeed = ( m_oi->GetRawButton( LEFT_TRIGGER_DRIVE_NEGATE ) ) ? -leftSpeed : leftSpeed;
    // TODO:  Instead of setting the rightSpeed to 0.0, read the RIGHT_TRIGGER_DRIVE_CONTROL raw axis value &
    //        RIGHT_TRIGGER_DRIVE_NEGATE to determine whether to negate the value or not
    float rightSpeed = 0.0;

    // Set the motors
    m_chassis->SetMotorSpeeds( leftSpeed, rightSpeed );
    return  false;
}


//--------------------------------------------------------------------
// Method:      GetIdentifier
// Description: This gets the name that identifies the drive mode..
// Returns:     const char*         identifier for the drive
//--------------------------------------------------------------------
const char* TriggerDrive::GetIdentifier() const
{
    return "Trigger Drive";
}
